package com.afp.ordermanagement.reactivekafkaservice;

import com.afp.ordermanagement.controller.OrderController;
import com.afp.ordermanagement.model.Item;
import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.model.OrderStatus;
import com.afp.ordermanagement.model.WarehouseOrder;
import com.afp.ordermanagement.repository.OrderRepository;
import com.afp.ordermanagement.service.EmailService;
import com.afp.ordermanagement.service.OrderService;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigurationPackage;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.receiver.KafkaReceiver;
import reactor.kafka.receiver.ReceiverRecord;

import java.util.*;

@Service
@Slf4j
public class Receiver {

    @Autowired
    private KafkaReceiver<String, String> kafkaReceiver;

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderRepository orderRepository;

    @Autowired
    private EmailService emailService;

    @EventListener(ApplicationStartedEvent.class)
    public void consumeOrderStatus() {
        kafkaReceiver.receive()
                .doOnNext(record -> System.out.println(record))
                .doOnNext(record -> log.info("record.value(): {} ", record.value()))
                .doOnNext(record -> updateOrderStatus(record.value()))
                .doOnError(throwable -> System.out.println(throwable.getMessage()))
                .subscribe();
    }




    private void updateOrderStatus(String orderStr) {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            WarehouseOrder order = objectMapper.readValue(orderStr, WarehouseOrder.class);
            log.info("order received from warehouse {}", order);
            String orderId = order.getOrderId();

            System.out.println("About to try sending an email.");
            Mono<Order> convertedOrder = orderService.getOrderById(order.getOrderId());
            convertedOrder.doOnSuccess(o -> {
                log.info("Existing order in the db: {}", o);
                o.setOrderStatus(OrderStatus.valueOf(order.getStatus()));
                emailService.emailCreator(o);
                orderService.updateOrderByOrderId(orderId, o).subscribe();
                System.out.println("ayayayayayayayaya receiver" + o);
            }).subscribe();

        } catch (Exception e) {
            log.error("Caught error on UpdateOrderStatus method", e);
            e.printStackTrace();
        }
    }




}
