package com.afp.ordermanagement.reactivekafkaservice;

import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.service.EmailService;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;
import reactor.kafka.sender.KafkaSender;
import reactor.kafka.sender.SenderRecord;
import reactor.kafka.sender.SenderResult;

import java.util.UUID;


@Service
@Slf4j
public class Sender {
    @Autowired
    private EmailService emailService;

    @Autowired
    private KafkaSender<String, Order> kafkaEventProducer;


    private static final String ORDER_TOPIC = "OMS_ORDER_UPDATE";

    private static final String WAREHOUSE_TOPIC = "WMOS_ORDER_UPDATE";

    public void sendOrderToTopic(Order orderObject) {
        log.info(String.format("##### -> Sender sending message: %s ", orderObject));
        ProducerRecord<String, Order> record = new ProducerRecord<>(ORDER_TOPIC, orderObject);
        System.out.println("In sendOrderToWarehouse");
        emailService.emailCreator(orderObject);
        Flux<SenderResult<UUID>> working = kafkaEventProducer.send(Mono.just(SenderRecord.create(record, UUID.randomUUID())))
                .doOnError(throwable -> System.out.println(throwable))
                .doOnNext(uuidSenderResult -> {
                    if (null != uuidSenderResult.exception()) {
                        System.out.println("working");
                    }
                });
        working.doOnError(throwable -> log.error("some error")).subscribe();
    }

    public void sendUpdatedOrderToWarehouseTopic(Order order) {
        ProducerRecord<String, Order> record = new ProducerRecord<>(WAREHOUSE_TOPIC, order);
        System.out.println("In sendUpdatedOrderToWarehouseTopic");
        Flux<SenderResult<Order>> updated = kafkaEventProducer.send(Mono.just(SenderRecord.create(record, order)))
                .doOnError(throwable -> System.out.println(throwable))
                .doOnNext(uuidSenderResult -> {
                    if (null != uuidSenderResult.exception()) {
                        System.out.println("send order update");
                    }
                });
        updated.doOnError(throwable -> log.error("error on sendUpdatedOrderToWarehouse method")).subscribe();
    }


}
